/*
gge.c
Author: Hayden Walles
Date: 9 September 2007
Methods of computing the energies of pixels in an image via an appproximation of the gradient.
*/

/*
    Copyright (C) 2007, 2008, 2009 Hayden Walles

    This file is part of Seamstress.

    Seamstress is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Seamstress is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "seams.h"

static int GGSetup=0;
static MATRIX *GGx,*GGy;
static int GGNorm;

int computeGaussianGradientEnergy(SEAM_COMMON *pic,void (*updater)(int done,int of,void *user),void *user,int *error){
  MATRIX *energy1,*energy2;
  int i,j;

  SET_ERROR(SEAMSTRESS_E_OK);

  energy1 = convolvePicture(pic,pic->luminance,GGx,error);
  if(energy1==NULL){
    return 0;
  }
  if(updater)
    updater(1,2,user);

  energy2 = convolvePicture(pic,pic->luminance,GGy,error);

  if(energy2==NULL){
    deleteMatrix(energy1);
    return 0;
  }
  if(updater)
    updater(2,2,user);
  /*Postprocess the convolved versions.*/
  
  for(i=0;i<matrixRows(energy1);i++){
    for(j=0;j<matrixCols(energy1);j++){
      setInteger(energy1,i,j,((abs(getInteger(energy1,i,j))+abs(getInteger(energy2,i,j)) /*+ /*experimental GGNorm/2-1*/)/GGNorm/2));
    }
  }
  deleteMatrix(energy2);


  pic->energy=energy1;
  
  
  return 1;

}


void gaussianGradientEnergyFinish(SEAM_COMMON *pic){

}

int gaussianGradientEnergyStart(SEAM_COMMON *pic, int *error){
  int hborder,vborder,maxhfilt,maxvfilt;

  /*Our job is to initialise any private energy stuff and to create the luminance matrix.  We only need the latter.*/

  maxhfilt=MAX(matrixCols(GGx),matrixCols(GGy));
  maxvfilt=MAX(matrixRows(GGx),matrixRows(GGy));
  
  hborder=maxhfilt/2;
  vborder=maxvfilt/2;

  if((pic->luminance=newMatrix(pic->height+2*vborder,pic->width+2*hborder,MT_INTEGER,error))==NULL){
    SET_ERROR(SEAMSTRESS_E_MEMORY);
    return 0;
  }

  SET_ERROR(SEAMSTRESS_E_OK);

  return 1;

}

int gaussianGradientEnergyInitialise(int *error){
  if(error!=NULL)
    *error=MATRIX_E_OK;

  if(GGSetup)
    return 1;
  
  GGx=createXGaussianGradient(1,&GGNorm,error);
  //experimental
  //GGNorm/=4;
  if(GGx==NULL)
    return 0;
  GGy=transpose(GGx,error); /*this gives the equivalent in the y direction.*/
  if(GGy==NULL){
    deleteMatrix(GGx);
    return 0;
  }
  GGSetup=1;
  
  return 1;
}


int recomputeGaussianGradientEnergy(SEAM_UNPICKER *pic,int *froms, int *tos){

  dualConvolvePictureInc(pic,pic->common.luminance,GGx,GGNorm,GGy,GGNorm,froms,tos,pic->common.energy);

  return 1;

}
